use actix_web::{post, web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;
use zxcvbn::{zxcvbn, Score};

use crate::{
    auth,
    models::{Registration, User},
    utils::{password_help, ApplicationError},
};

#[derive(Deserialize)]
struct RegisterForm {
    token: String,
    password: String,
    passwordretyped: String,
    dry: Option<bool>,
}

#[post("/register")]
async fn post(
    form: web::Form<RegisterForm>,
    pool: web::Data<PgPool>,
) -> Result<impl Responder, ApplicationError> {
    let is_dry = form.dry.unwrap_or(false);
    // TODO: flip unauthorized with not found or unwrap result in a other way
    let token = Registration::does_token_exist(pool.get_ref(), &form.token).await?.ok_or(ApplicationError::Unauthorized)?;

    if form.password.chars().count() > 256 {
        if is_dry {
            return Ok(HttpResponse::BadRequest().body("<div id=\"password-strength\" class=\"mb-3 help content is-danger\">Password darf nicht länger als 256 Zeichen sein.</div>"));
        } else {
            return Ok(HttpResponse::NoContent().finish());
        }
    }

    let user = User::read_by_id(pool.get_ref(), token.userid).await?;
    let mut split_names: Vec<&str> = user.name.as_str().split_whitespace().collect();
    let mut user_inputs = vec![user.email.as_str()];
    user_inputs.append(&mut split_names);
    let entropy = zxcvbn(&form.password, &user_inputs);

    if entropy.score() < Score::Three {
        if is_dry {
            let message = password_help::generate_for_entropy(&entropy);

            return Ok(HttpResponse::BadRequest().body(message));
        } else {
            return Ok(HttpResponse::NoContent().finish());
        }
    }

    if is_dry {
        if entropy.score() == Score::Three {
            return Ok(HttpResponse::Ok()
                    .body("<div id=\"password-strength\" class=\"mb-3 help content is-success\">Sicheres Passwort.</div>"));
        } else {
            return Ok(HttpResponse::Ok()
                    .body("<div id=\"password-strength\" class=\"mb-3 help content is-success\">Sehr sicheres Passwort.</div>"));
        }
    }

    if form.password != form.passwordretyped {
        return Ok(HttpResponse::BadRequest().body("Passwörter stimmen nicht überein!"));
    }
    let (hash, salt) = auth::utils::generate_salt_and_hash_plain_password(&form.password).unwrap();

    User::update(
        pool.get_ref(),
        token.userid,
        None,
        None,
        Some(&hash),
        Some(&salt),
        None,
        None,
        None,
        None,
        None,
    )
    .await?;

    Registration::delete(pool.get_ref(), &token.token).await?;

    Ok(HttpResponse::Ok().body(r#"<div class="block">Registrierung abgeschlossen.</div><a class="block button is-primary" hx-boost="true" href="/login">Zum Login</a>"#))
}
